(* $Id: pax_types.mli 799 2008-06-03 15:13:33Z gerd $ 
 * ----------------------------------------------------------------------
 * Copyright 2008 by Gerd Stolpmann.
 *
 * This file is part of godi-tools.
 *
 * godi-tools is free software; you can redistribute it and/or modify it
 * under the conditions in the file LICENSE.
 *
 * godi-tools is distributed WITHOUT ANY WARRANTY, as detailed in the
 * file LICENSE.
 *)

class type file =
object
  method read_file : unit -> Netchannels.in_obj_channel
    (* Opens the file for reading. *)

  method create_file : unit -> Netchannels.out_obj_channel
    (* Creates the file, and opens it for writing. It is an error
       if the file already exists.
     *)

  (* Note that the channels will never be closed! The user has to do
     this.
   *)

  (* We currently don't support appending to files. This would be 
     another method. Appending requires the ability to seek, and
     is a bit more complicated.
   *)
end
  (* A file object is connected with a file or pipe. If seeking is supported,
     the methods can be called several times
   *)


type filetype =
    [ `Regular
    | `Directory
	(* for directories, the meaning of [size] is unspecified *)
    | `Hardlink of string
	(* a hard link to another (regular) file *)
    | `Symlink of string
	(* a symbolic link to another file or directory *)
    | `Block of int * int
	(* block special file with major/minor identity *)
    | `Char of int * int
	(* character special file with major/minor identity *)
    | `Fifo
	(* fifo special file *)
    | `Other of string * string
	(* Other format. The first string defines the file format, and the
           second the type in this format. We use:
           - ("ustar", s) where s is a one-character string with the file type
           - ("oldgnu", s) where s is a one-character string with the file type
	 *)
    ]

class type metadata =
object
  method name : string
    (* The name of the file or directory or special file *)
  method mode : int
    (* file mode as in Unix *)
  method size : int64
    (* file size *)
  method owner_uid : int option
  method owner_uname : string option
  method owner_gid : int option
  method owner_gname : string option
    (* User and group of the owner. May only be partially available *)
  method mtime : int64
    (* time of last modification *)
  method filetype : filetype
    (* The type of the file *)
  method format : string option
    (* The format of the entry. When writing [None] means to use the
       default format
     *)
  method have_data : bool
    (* If true, we have data. The amount of data is [size]. If [false]
       there is no data, and [size] must be ignored.

       This value is ignored when writing elements.
     *)
end



type archive_rd_file =
    [ `Channel of Netchannels.in_obj_channel
    | `File of Godi_file.filename * Godi_file.file_id
    ]


class type archive_rd =
object
  (* A reader iterates over the contents. *)

  method get_metadata : metadata
    (* Get the metadata of the file at the current position *)

  method get_data : unit ->  archive_rd_file
    (* Reads the file. It is allowed not to read from the channel until the
       end. However, the channel will be closed when the [next] method is
       invoked.

       The file_id is returned if the channel is connected with a real
       file (only used by the copier). The string is the file name.
     *)

  method next : unit -> archive_rd option
    (* Get the next element of the archive. Returns [None] if there is no
       next element
     *)
end


class type archive_wr =
object
  (* A writer appends to the contents *)

  method put_data : metadata -> Netchannels.out_obj_channel
    (* Puts an element w/ data. The metadata object announces a certain length
       of the element, and exactly this number of bytes must be written to
       the channel. The channel must be closed then. After that it is allowed
       to put further elements.
     *)

  method put_metadata_only : metadata -> unit
    (* Puts an element with only metadata, and no data *)

  method close : unit -> unit
    (* Finishes writing to the archive *)

end


class type archive =
object
  method archive_type : string
    (* The type of the archive as string, e.g. "tar" *)

  method read_archive : unit -> archive_rd option
    (* Start reading from the archive. The possible operations are made
       available via the archive_rd object. Returns [None] if the archive
       is empty.
     *)

  method create_archive : unit -> archive_wr
    (* Create a new archive, and start appending to it. The operations are
       in the archive_wr object
     *)
end
